@@ -1,4 +1,5 @@
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 #include "fuzz_common_code.h"
 
 #include <stdint.h>
@@ -7,27 +8,13 @@
 struct ndpi_detection_module_struct *ndpi_info_mod = NULL;
 struct ndpi_flow_struct *flow = NULL;
 
-extern const uint8_t *get_crypto_data(struct ndpi_detection_module_struct *ndpi_struct,
-				      struct ndpi_flow_struct *flow,
-				      u_int8_t *clear_payload, uint32_t clear_payload_len,
-				      uint64_t *crypto_data_len);
-extern void process_tls(struct ndpi_detection_module_struct *ndpi_struct,
-			struct ndpi_flow_struct *flow,
-			const u_int8_t *crypto_data, uint32_t crypto_data_len,
-			uint32_t version);
-extern void process_chlo(struct ndpi_detection_module_struct *ndpi_struct,
-			 struct ndpi_flow_struct *flow,
-			 const u_int8_t *crypto_data, uint32_t crypto_data_len);
-extern int is_version_with_tls(uint32_t version);
-
-
 int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   const u_int8_t *crypto_data;
   uint64_t crypto_data_len;
   u_int32_t first_int, version = 0;
 
   if(ndpi_info_mod == NULL) {
-    fuzz_init_detection_module(&ndpi_info_mod);
+    fuzz_init_detection_module(&ndpi_info_mod, NULL);
 
     flow = ndpi_calloc(1, SIZEOF_FLOW_STRUCT);
   }
@@ -56,7 +43,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     if(!is_version_with_tls(version)) {
       process_chlo(ndpi_info_mod, flow, crypto_data, crypto_data_len);
     } else {
-      process_tls(ndpi_info_mod, flow, crypto_data, crypto_data_len, version);
+      process_tls(ndpi_info_mod, flow, crypto_data, crypto_data_len);
     }
   }
 
