@@ -29,6 +29,7 @@
 #include <unistd.h>
 
 #include <cstring>
+#include <fstream>
 #include <list>
 
 #include "INIReader.h"
@@ -36,51 +37,14 @@
 #include "nd-config.hpp"
 #include "nd-except.hpp"
 #include "nd-plugin.hpp"
+#include "nd-protos.hpp"
 #include "nd-util.hpp"
 #include "netifyd.hpp"
 
 using namespace std;
+using json = nlohmann::json;
 
-ndGlobalConfig::ndGlobalConfig()
-  : napi_tls_verify(true), napi_vendor(ND_API_VENDOR),
-    path_agent_status(ND_AGENT_STATUS_PATH),
-    path_app_config(ND_CONF_APP_PATH),
-    path_cat_config(ND_CONF_CAT_PATH),
-    path_categories(ND_CATEGORIES_PATH),
-    path_config(ND_CONF_FILE_NAME),
-    path_functions(ND_FUNCTIONS_PATH),
-    path_interfaces(ND_INTERFACES_PATH),
-    path_legacy_config(ND_CONF_LEGACY_PATH),
-    path_pid_file(ND_PID_FILE_NAME), path_plugins(ND_PLUGINS_PATH),
-    path_plugin_libdir(ND_PLUGIN_LIBDIR),
-    path_shared_data(ND_SHARED_DATADIR),
-    path_state_persistent(ND_PERSISTENT_STATEDIR),
-    path_state_volatile(ND_VOLATILE_STATEDIR),
-    path_uuid(ND_AGENT_UUID_PATH),
-    path_uuid_serial(ND_AGENT_SERIAL_PATH),
-    path_uuid_site(ND_SITE_UUID_PATH),
-    url_napi_bootstrap(ND_URL_API_BOOTSTRAP),
-    dhc_storage(ndDHCStorage::PERSISTENT),
-    fhc_storage(ndFHCStorage::PERSISTENT),
-    capture_type(ndCaptureType::NONE),
-    capture_read_timeout(ND_CAPTURE_READ_TIMEOUT),
-    h_flow(stderr), ca_capture_base(0), ca_conntrack(-1),
-    ca_detection_base(0), ca_detection_cores(-1),
-    max_packet_queue(ND_MAX_PKT_QUEUE_KB * 1024),
-    max_capture_length(ND_PCAP_SNAPLEN),
-    flags(ndGlobalFlags::NONE), verbosity(0),
-    verbosity_flags(ndVerbosityFlags::EVENT_DPI_COMPLETE),
-    fhc_purge_divisor(ND_FHC_PURGE_DIVISOR),
-    fm_buckets(ND_FLOW_MAP_BUCKETS),
-    max_detection_pkts(ND_MAX_DETECTION_PKTS),
-    max_dhc(ND_MAX_DHC_ENTRIES),
-    max_fhc(ND_MAX_FHC_ENTRIES), max_flows(0),
-    ttl_capture_delay(0), ttl_idle_flow(ND_TTL_IDLE_FLOW),
-    ttl_idle_tcp_flow(ND_TTL_IDLE_TCP_FLOW),
-    ttl_napi_tick(ND_TTL_API_TICK),
-    ttl_napi_update(ND_TTL_API_UPDATE), update_imf(1),
-    update_interval(ND_STATS_INTERVAL), reader(nullptr) {
-    flags |= ndGlobalFlags::SSL_VERIFY;
+ndGlobalConfig::ndGlobalConfig() {
     flags |= ndGlobalFlags::AUTO_FLOW_EXPIRY;
 #ifdef _ND_ENABLE_CONNTRACK
     flags |= ndGlobalFlags::USE_CONNTRACK;
@@ -119,14 +83,14 @@ void ndGlobalConfig::Close(void) {
     }
 }
 
-bool ndGlobalConfig::Load(const string &filename) {
+bool ndGlobalConfig::Load(const string &filename, bool profile) {
     struct stat extern_config_stat;
     if (stat(filename.c_str(), &extern_config_stat) < 0) {
         if (errno == ENOENT) {
             fprintf(stderr,
               "Configuration file not found: %s\n",
               filename.c_str());
-            return true;
+            return false;
         }
         fprintf(stderr,
           "Can not stat configuration file: %s: %s\n",
@@ -159,7 +123,8 @@ bool ndGlobalConfig::Load(const string &filename) {
 
     // Netify section
     ndGC_SetFlag(ndGlobalFlags::AUTO_INFORMATICS,
-      r->GetBoolean("netifyd", "auto_informatics", false));
+      r->GetBoolean(ND_NAME_SHORT, "auto_informatics",
+        (! profile) ? false : ndGC_AUTO_INFORMATICS));
 
     if (uuid.empty())
         uuid = r->Get(ND_NAME_SHORT, "uuid", ND_AGENT_UUID_NULL);
@@ -172,173 +137,197 @@ bool ndGlobalConfig::Load(const string &filename) {
         uuid_site = r->Get(ND_NAME_SHORT, "uuid_site",
           ND_SITE_UUID_NULL);
 
-    path_state_persistent = r->Get("netifyd",
-      "path_state_persistent", ND_PERSISTENT_STATEDIR);
+    if (network_namespace.empty())
+        network_namespace = r->Get(ND_NAME_SHORT, "network_namespace", "");
 
-    path_state_volatile = r->Get("netifyd",
-      "path_state_volatile", ND_VOLATILE_STATEDIR);
+    if (! profile) {
+        path_state_persistent = r->Get(ND_NAME_SHORT,
+          "path_state_persistent", ND_PERSISTENT_STATEDIR);
 
-    path_shared_data = r->Get("netifyd", "path_shared_data",
-      ND_SHARED_DATADIR);
+        path_profile = r->Get(ND_NAME_SHORT, "profile", "");
 
-    UpdatePaths();
-    UpdateConfigVars();
+        path_state_volatile = r->Get(ND_NAME_SHORT,
+          "path_state_volatile", ND_VOLATILE_STATEDIR);
 
-    value = r->Get("netifyd", "path_pid_file",
-      path_state_volatile + "/" + ND_PID_FILE_BASE);
-    nd_expand_variables(value, path_pid_file, conf_vars);
+        path_shared_data = r->Get(ND_NAME_SHORT,
+          "path_shared_data", ND_SHARED_DATADIR);
 
-    value = r->Get("netifyd", "path_plugins",
-      path_state_persistent + "/" + ND_PLUGINS_BASE);
-    nd_expand_variables(value, path_plugins, conf_vars);
+        UpdatePaths();
+        UpdateConfigVars();
 
-    value = r->Get("netifyd", "path_plugin_libdir", ND_PLUGIN_LIBDIR),
-    nd_expand_variables(value, path_plugin_libdir, conf_vars);
+        value = r->Get(ND_NAME_SHORT, "path_pid_file",
+          path_state_volatile + "/" + ND_PID_FILE_BASE);
+        nd_expand_variables(value, path_pid_file, conf_vars);
 
-    value = r->Get("netifyd", "path_categories",
-      path_state_persistent + "/" + ND_CATEGORIES_BASE);
-    nd_expand_variables(value, path_categories, conf_vars);
+        value = r->Get(ND_NAME_SHORT, "path_plugins",
+          path_state_persistent + "/" + ND_PLUGINS_BASE);
+        nd_expand_variables(value, path_plugins, conf_vars);
 
-    value = r->Get("netifyd", "path_interfaces",
-      path_state_persistent + "/" + ND_INTERFACES_BASE);
-    nd_expand_variables(value, path_interfaces, conf_vars);
+        value = r->Get(ND_NAME_SHORT, "path_plugin_libdir",
+          ND_PLUGIN_LIBDIR),
+        nd_expand_variables(value, path_plugin_libdir, conf_vars);
 
-    path_uuid = r->Get("netifyd", "path_uuid",
-      path_state_persistent + "/" + ND_AGENT_UUID_BASE);
+        value = r->Get(ND_NAME_SHORT, "path_categories",
+          path_state_persistent + "/" + ND_CATEGORIES_BASE);
+        nd_expand_variables(value, path_categories, conf_vars);
 
-    path_uuid_serial = r->Get("netifyd", "path_uuid_serial",
-      path_state_persistent + "/" + ND_AGENT_SERIAL_BASE);
+        value = r->Get(ND_NAME_SHORT, "path_interfaces",
+          path_state_persistent + "/" + ND_INTERFACES_BASE);
+        nd_expand_variables(value, path_interfaces, conf_vars);
 
-    path_uuid_site = r->Get("netifyd", "path_uuid_site",
-      path_state_persistent + "/" + ND_SITE_UUID_BASE);
+        path_uuid = r->Get(ND_NAME_SHORT, "path_uuid",
+          path_state_persistent + "/" + ND_AGENT_UUID_BASE);
 
-    UpdateConfigVars();
+        path_uuid_serial = r->Get(ND_NAME_SHORT,
+          "path_uuid_serial",
+          path_state_persistent + "/" + ND_AGENT_SERIAL_BASE);
 
-    update_interval = (unsigned)r->GetInteger("netifyd",
-      "update_interval", ND_STATS_INTERVAL);
+        path_uuid_site = r->Get(ND_NAME_SHORT,
+          "path_uuid_site",
+          path_state_persistent + "/" + ND_SITE_UUID_BASE);
 
-    max_packet_queue =
-      r->GetInteger("netifyd", "max_packet_queue_kb",
-        ND_MAX_PKT_QUEUE_KB) *
-      1024;
+        UpdateConfigVars();
+    }
+    else path_profile.clear();
 
-    ndGC_SetFlag(ndGlobalFlags::SSL_VERIFY,
-      r->GetBoolean("netifyd", "ssl_verify", true));
+    update_interval = (unsigned)r->GetInteger(ND_NAME_SHORT,
+      "update_interval",
+      (! profile) ? ND_STATS_INTERVAL : update_interval);
 
-    ndGC_SetFlag(ndGlobalFlags::SSL_USE_TLSv1,
-      r->GetBoolean("netifyd", "ssl_use_tlsv1", false));
+    max_capture_length = (uint16_t)r->GetInteger(ND_NAME_SHORT,
+      "max_capture_length",
+      (! profile) ? ND_PCAP_SNAPLEN : max_capture_length);
 
-    max_capture_length = (uint16_t)r->GetInteger("netifyd",
-      "max_capture_length", ND_PCAP_SNAPLEN);
-
-    max_detection_pkts = (unsigned)r->GetInteger("netifyd",
-      "max_detection_pkts", ND_MAX_DETECTION_PKTS);
+    max_detection_pkts = (unsigned)r->GetInteger(ND_NAME_SHORT,
+      "max_detection_pkts",
+      (! profile) ? ND_MAX_DETECTION_PKTS : max_detection_pkts);
 
     ndGC_SetFlag(ndGlobalFlags::SYN_SCAN_PROTECTION,
-      r->GetBoolean("netifyd", "syn_scan_protection", false));
+      r->GetBoolean(ND_NAME_SHORT, "syn_scan_protection",
+        (! profile) ? false : ndGC_SYN_SCAN_PROTECTION));
     ndGC_SetFlag(ndGlobalFlags::AUTO_FLOW_EXPIRY,
-      r->GetBoolean("netifyd", "auto_flow_expiry", true));
+      r->GetBoolean(ND_NAME_SHORT, "auto_flow_expiry",
+        (! profile) ? true : ndGC_AUTO_FLOW_EXPIRY));
 
-    ttl_idle_flow = (unsigned)r->GetInteger("netifyd",
-      "ttl_idle_flow", ND_TTL_IDLE_FLOW);
-    ttl_idle_tcp_flow = (unsigned)r->GetInteger("netifyd",
-      "ttl_idle_tcp_flow", ND_TTL_IDLE_TCP_FLOW);
+    ttl_idle_flow = (unsigned)r->GetInteger(ND_NAME_SHORT,
+      "ttl_idle_flow",
+      (! profile) ? ND_TTL_IDLE_FLOW : ttl_idle_flow);
+    ttl_idle_tcp_flow = (unsigned)r->GetInteger(ND_NAME_SHORT,
+      "ttl_idle_tcp_flow",
+      (! profile) ? ND_TTL_IDLE_TCP_FLOW : ttl_idle_flow);
 
-    max_flows = (size_t)r->GetInteger("netifyd", "max_flows", 0);
+    max_flows = (size_t)r->GetInteger(ND_NAME_SHORT,
+      "max_flows", (! profile) ? 0 : max_flows);
 
     ndGC_SetFlag(ndGlobalFlags::SOFT_DISSECTORS,
-      r->GetBoolean("netifyd", "soft_dissectors", true));
+      r->GetBoolean(ND_NAME_SHORT, "soft_dissectors",
+        (! profile) ? true : ndGC_SOFT_DISSECTORS));
 
     ndGC_SetFlag(ndGlobalFlags::DOTD_CATEGORIES,
-      r->GetBoolean("netifyd", "dotd_categories", true));
+      r->GetBoolean(ND_NAME_SHORT, "dotd_categories",
+        (! profile) ? true : ndGC_DOTD_CATEGORIES));
 
-    fm_buckets = (unsigned)r->GetInteger("netifyd",
-      "flow_map_buckets", ND_FLOW_MAP_BUCKETS);
+    ndGC_SetFlag(ndGlobalFlags::ENABLE_COREDUMPS,
+      r->GetBoolean(ND_NAME_SHORT, "enable_coredumps",
+        (! profile) ? true : ndGC_ENABLE_COREDUMPS));
+
+    fm_buckets = (unsigned)r->GetInteger(ND_NAME_SHORT,
+      "flow_map_buckets",
+      (! profile) ? ND_FLOW_MAP_BUCKETS : fm_buckets);
 
     ndGC_SetFlag(ndGlobalFlags::USE_GETIFADDRS,
       r->GetBoolean(ND_NAME_SHORT, "use_getifaddrs",
 #if defined(__FreeBSD__)
-        true
+        (! profile) ? true : ndGC_USE_GETIFADDRS
 #else
-        false
+        (! profile) ? false : ndGC_USE_GETIFADDRS
 #endif
         ));
 
+    user_id = r->Get(ND_NAME_SHORT, "user", "");
+    group_id = r->Get(ND_NAME_SHORT, "group", "");
+
     // Threading section
     ca_capture_base = (int16_t)r->GetInteger("threads",
       "capture_base", this->ca_capture_base);
-    ca_conntrack = (int16_t)r->GetInteger("threads",
-      "conntrack", this->ca_conntrack);
     ca_detection_base = (int16_t)r->GetInteger("threads",
       "detection_base", this->ca_detection_base);
     ca_detection_cores = (int16_t)r->GetInteger("threads",
       "detection_cores", this->ca_detection_cores);
 
     // Capture defaults section
-    capture_read_timeout = (unsigned)r->GetInteger(
-      "capture-defaults", "read_timeout", ND_CAPTURE_READ_TIMEOUT);
-    capture_type = LoadCaptureType(reader,
-      "capture-defaults", "capture_type");
+    if (! profile) {
+        capture_read_timeout = (unsigned)r->GetInteger(
+          "capture-defaults", "read_timeout", ND_CAPTURE_READ_TIMEOUT);
+        capture_type = LoadCaptureType(reader,
+          "capture-defaults", "capture_type");
 
-    if (capture_type == ndCaptureType::NONE) {
+        if (capture_type == ndCaptureType::NONE) {
 #if defined(_ND_ENABLE_LIBPCAP)
-        capture_type = ndCaptureType::PCAP;
+            capture_type = ndCaptureType::PCAP;
 #elif defined(_ND_ENABLE_TPACKETV3)
-        capture_type = ndCaptureType::TPV3;
+            capture_type = ndCaptureType::TPV3;
 #elif defined(_ND_ENABLE_NFQUEUE)
-        capture_type = ndCaptureType::TPV3;
+            capture_type = ndCaptureType::TPV3;
 #else
-        fprintf(stderr,
-          "Not default capture type could be "
-          "determined.\n");
-        return false;
+            fprintf(stderr,
+              "Not default capture type could be "
+              "determined.\n");
+            return false;
 #endif
-    }
+        }
 
-    // TPv3 capture defaults section
-    ndFlags<ndCaptureType> ct = ndCaptureType::TPV3;
-    if (
-      ! LoadCaptureSettings(reader, "capture-defaults-tpv3",
-        ct, static_cast<void *>(&tpv3_defaults)))
-        return false;
+        // TPv3 capture defaults section
+        ndFlags<ndCaptureType> ct = ndCaptureType::TPV3;
+        if (! LoadCaptureSettings(reader,
+              "capture-defaults-tpv3", ct,
+              static_cast<void *>(&tpv3_defaults)))
+            return false;
+    }
 
     // Flow Hash Cache section
     ndGC_SetFlag(ndGlobalFlags::USE_FHC,
-      r->GetBoolean("flow-hash-cache", "enable", true));
+      r->GetBoolean("flow-hash-cache", "enable",
+        (! profile) ? true : ndGC_USE_FHC));
 
-    string fhc_storage_mode = r->Get("flow-hash-cache",
-      "save", "persistent");
+    if (! profile) {
+        string fhc_storage_mode = r->Get("flow-hash-cache",
+          "save", "persistent");
 
-    if (fhc_storage_mode == "persistent")
-        fhc_storage = ndFHCStorage::PERSISTENT;
-    else if (fhc_storage_mode == "volatile")
-        fhc_storage = ndFHCStorage::VOLATILE;
-    else fhc_storage = ndFHCStorage::DISABLED;
+        if (fhc_storage_mode == "persistent")
+            fhc_storage = ndFHCStorage::PERSISTENT;
+        else if (fhc_storage_mode == "volatile")
+            fhc_storage = ndFHCStorage::VOLATILE;
+        else fhc_storage = ndFHCStorage::DISABLED;
+    }
 
     max_fhc = (size_t)r->GetInteger("flow-hash-cache",
-      "cache_size", ND_MAX_FHC_ENTRIES);
-    fhc_purge_divisor = (size_t)r->GetInteger(
-      "flow-hash-cache", "purge_divisor", ND_FHC_PURGE_DIVISOR);
+      "cache_size", (! profile) ? ND_MAX_FHC_ENTRIES : max_fhc);
 
     // DNS Cache section
     ndGC_SetFlag(ndGlobalFlags::USE_DHC,
-      r->GetBoolean("dns-hint-cache", "enable", true));
+      r->GetBoolean("dns-hint-cache", "enable",
+        (! profile) ? true : ndGC_USE_DHC));
     ndGC_SetFlag(ndGlobalFlags::DHC_PARTIAL_LOOKUPS,
-      r->GetBoolean("dns-hint-cache", "partial_lookups", false));
+      r->GetBoolean("dns-hint-cache", "partial_lookups",
+        (! profile) ? false : ndGC_DHC_PARTIAL_LOOKUPS));
 
-    string dhc_storage_mode = r->Get("dns-hint-cache",
-      "save", "persistent");
+    if (! profile) {
+        string dhc_storage_mode = r->Get("dns-hint-cache",
+          "save", "persistent");
 
-    if (dhc_storage_mode == "persistent" ||
-      dhc_storage_mode == "1" || dhc_storage_mode == "yes" ||
-      dhc_storage_mode == "true")
-        dhc_storage = ndDHCStorage::PERSISTENT;
-    else if (dhc_storage_mode == "volatile")
-        dhc_storage = ndDHCStorage::VOLATILE;
-    else dhc_storage = ndDHCStorage::DISABLED;
+        if (dhc_storage_mode == "persistent" ||
+          dhc_storage_mode == "1" ||
+          dhc_storage_mode == "yes" ||
+          dhc_storage_mode == "true")
+            dhc_storage = ndDHCStorage::PERSISTENT;
+        else if (dhc_storage_mode == "volatile")
+            dhc_storage = ndDHCStorage::VOLATILE;
+        else dhc_storage = ndDHCStorage::DISABLED;
+    }
 
     max_dhc = (unsigned)r->GetInteger("dns-hint-cache",
-      "cache-size", ND_MAX_DHC_ENTRIES);
+      "cache-size", (! profile) ? ND_MAX_DHC_ENTRIES : max_dhc);
 
     // Privacy filter section
     for (int i = 0;; i++) {
@@ -347,7 +336,7 @@ bool ndGlobalConfig::Load(const string &filename) {
         string mac_addr = r->Get("privacy-filter", os.str(),
           "");
 
-        if (mac_addr.size() == 0) break;
+        if (mac_addr.empty()) break;
 
         uint8_t mac[ETH_ALEN];
         if (nd_string_to_mac(mac_addr, mac)) {
@@ -363,7 +352,7 @@ bool ndGlobalConfig::Load(const string &filename) {
         string host_addr = r->Get("privacy-filter",
           os.str(), "");
 
-        if (host_addr.size() == 0) break;
+        if (host_addr.empty()) break;
 
         struct addrinfo hints;
         struct addrinfo *result, *rp;
@@ -401,8 +390,7 @@ bool ndGlobalConfig::Load(const string &filename) {
         string replace = r->Get("privacy-filter", os.str(),
           "");
 
-        if (search.size() == 0 || replace.size() == 0)
-            break;
+        if (search.empty() || replace.empty()) break;
 
         try {
             regex *rx_search = new regex(search,
@@ -422,30 +410,53 @@ bool ndGlobalConfig::Load(const string &filename) {
 
     ndGC_SetFlag(ndGlobalFlags::PRIVATE_EXTADDR,
       r->GetBoolean("privacy-filter",
-        "private_external_addresses", false));
+        "private_external_addresses",
+        (! profile) ? false : ndGC_PRIVATE_EXTADDR));
 
     // Netify API section
     ndGC_SetFlag(ndGlobalFlags::USE_NAPI,
-      r->GetBoolean("netify-api", "enable", false));
+      r->GetBoolean("netify-api", "enable",
+        (! profile) ? false : ndGC_USE_NAPI));
     ttl_napi_tick = r->GetInteger("netify-api",
-      "tick_interval", ND_TTL_API_TICK);
+      "update_tick", (! profile) ? ND_TTL_API_TICK : ttl_napi_tick);
     ttl_napi_update = r->GetInteger("netify-api",
-      "update_interval", ND_TTL_API_UPDATE);
+      "update_interval",
+      (! profile) ? ND_TTL_API_UPDATE : ttl_napi_update);
     url_napi_bootstrap = r->Get("netify-api",
-      "bootstrap_url", ND_URL_API_BOOTSTRAP);
+      "bootstrap_url",
+      (! profile) ? ND_URL_API_BOOTSTRAP : url_napi_bootstrap);
     napi_vendor = r->Get("netify-api", "vendor", ND_API_VENDOR);
     napi_tls_verify = r->GetBoolean("netify-api",
-      "tls_verify", true);
+      "tls_verify", (! profile) ? true : napi_tls_verify);
 
     // Protocols section
-    r->GetSection("protocols", protocols);
+    Protocols p;
+    r->GetSection("protocols", p);
 
-    // Add plugins
-    vector<string> files;
-    if (nd_scan_dotd(path_plugins, files)) {
-        for (auto &f : files) {
-            if (! AddPlugin(path_plugins + "/" + f))
-                return false;
+    for (auto &i : p) {
+        auto r = protocols.insert(i);
+        if (r.second) continue;
+        protocols[i.first] = i.second;
+    }
+
+    LoadProtocols();
+
+    // Load plugins
+
+    if (! profile) {
+        // Add plugins
+        vector<string> files;
+        if (nd_scan_dotd(path_plugins, files)) {
+            for (auto &f : files) {
+                if (! AddPlugin(path_plugins + "/" + f))
+                    return false;
+            }
+        }
+
+        // Optionally, load profile configuration...
+        if (! path_profile.empty()) {
+            Close();
+            return Load(path_profile, true);
         }
     }
 
@@ -596,7 +607,7 @@ bool ndGlobalConfig::ForceReset(void) {
           "be completed.\n");
     }
 
-    if (result.size())
+    if (! result.empty())
         fprintf(stdout, "%s", result.c_str());
 
     if (rc == 0) fprintf(stdout, "Reset successful.\n");
@@ -604,6 +615,59 @@ bool ndGlobalConfig::ForceReset(void) {
     return success;
 }
 
+bool ndGlobalConfig::LoadProtocols(void) {
+    if (! nd_file_exists(path_proto_config)) return false;
+
+    ifstream ifs(path_proto_config);
+    if (! ifs.is_open()) {
+        nd_dprintf("error opening protocols config: %s\n",
+            path_proto_config.c_str());
+        return false;
+    }
+
+    json jprotos;
+
+    try {
+        ifs >> jprotos;
+    }
+    catch (exception &e) {
+        nd_printf(
+          "Error loading protocols: %s: JSON parse error\n",
+          path_proto_config.c_str());
+        nd_dprintf("%s: %s\n", path_proto_config.c_str(), e.what());
+
+        return false;
+    }
+
+    unsigned count = 0;
+    auto jp = jprotos.find("protocols");
+    if (jp == jprotos.end()) {
+        nd_dprintf("%s: invalid schema: %s not found.\n",
+            path_proto_config.c_str(), "protocols");
+        return false;
+    }
+
+    for (auto &p : jp->get<json::array_t>()) {
+
+        auto jid = p.find("id");
+        if (jid == p.end() || ! jid->is_number())
+            continue;
+
+        auto jenable = p.find("enable");
+        if (jenable == p.end() || ! jenable->is_boolean())
+            continue;
+
+        auto tag = ndProto::Tags.find(
+            static_cast<ndProto::Id>(jid->get<unsigned>())
+        );
+
+        protocols[tag->second] = (jenable->get<bool>()) ? "include" : "exclude";
+        count++;
+    }
+
+    return (count > 0);
+}
+
 bool ndGlobalConfig::LoadInterfaces(const string &filename) {
     if (ndGC_IGNORE_IFACE_CONFIGS) return true;
 
@@ -833,22 +897,11 @@ bool ndGlobalConfig::LoadInterfaces(void *config_reader) {
             }
             break;
         case ndCaptureType::NFQ:
-            p = iface.find_first_of("0123456789");
-            if (p == string::npos ||
-              strncasecmp(iface.c_str(), "nfq", 3))
-            {
-                fprintf(stderr,
-                  "Invalid NFQUEUE identifier: %s\n",
-                  iface.c_str());
-                return false;
-            }
             config = static_cast<void *>(new nd_config_nfq);
             if (! LoadCaptureSettings(r, s, type, config)) {
                 delete static_cast<nd_config_nfq *>(config);
                 return false;
             }
-            static_cast<nd_config_nfq *>(config)->queue_id =
-              (unsigned)strtol(iface.substr(p).c_str(), NULL, 0);
             break;
         default: break;
         }
@@ -860,18 +913,18 @@ bool ndGlobalConfig::LoadInterfaces(void *config_reader) {
             os << "address[" << i << "]";
             string addr = r->Get(s, os.str(), "");
 
-            if (addr.size() == 0) break;
+            if (addr.empty()) break;
 
             AddInterfaceAddress(iface, addr);
         }
 
         string peer = r->Get(s, "peer", "");
 
-        if (peer.size()) AddInterfacePeer(iface, peer);
+        if (! peer.empty()) AddInterfacePeer(iface, peer);
 
         string filter = r->Get(s, "filter", "");
 
-        if (filter.size())
+        if (! filter.empty())
             AddInterfaceFilter(iface, filter);
     }
 
@@ -881,7 +934,7 @@ bool ndGlobalConfig::LoadInterfaces(void *config_reader) {
 void ndGlobalConfig::ClearInterfaces(bool cmdline_entries) {
     list<string> clear_list;
 
-    if (interfaces.size()) {
+    if (! interfaces.empty()) {
         for (auto &r : interfaces) {
             for (auto &i : r.second) {
                 if (((i.second.first & ndCaptureType::CMDLINE) ==
@@ -972,7 +1025,10 @@ bool ndGlobalConfig::LoadCaptureSettings(void *config_reader,
   const string &section, ndFlags<ndCaptureType> &type,
   void *config) const {
     INIReader *r = static_cast<INIReader *>(config_reader);
-
+#ifdef _ND_ENABLE_CONNTRACK_COUNTERS
+    bool conntrack_counters = r->GetBoolean(section,
+      "conntrack_counters", false);
+#endif
     if (ndCT_TYPE(type.flags) == ndCaptureType::PCAP) {
         nd_config_pcap *pcap = static_cast<nd_config_pcap *>(config);
         string capture_filename = r->Get(section,
@@ -992,8 +1048,13 @@ bool ndGlobalConfig::LoadCaptureSettings(void *config_reader,
             else
                 type = ndCaptureType::PCAP_OFFLINE |
                   ndCaptureType::CMDLINE;
+
             pcap->capture_filename = capture_filename;
         }
+#ifdef _ND_ENABLE_CONNTRACK_COUNTERS
+        if (! ((type & ndCaptureType::PCAP_OFFLINE) == ndCaptureType::PCAP_OFFLINE))
+            pcap->conntrack_counters = conntrack_counters;
+#endif
     }
     else if (ndCT_TYPE(type.flags) == ndCaptureType::TPV3) {
         nd_config_tpv3 *tpv3 = static_cast<nd_config_tpv3 *>(config);
@@ -1056,11 +1117,19 @@ bool ndGlobalConfig::LoadCaptureSettings(void *config_reader,
 
         tpv3->rb_blocks = (unsigned)r->GetInteger(section,
           "rb_blocks", tpv3_defaults.rb_blocks);
+#ifdef _ND_ENABLE_CONNTRACK_COUNTERS
+        tpv3->conntrack_counters = conntrack_counters;
+#endif
     }
     else if (ndCT_TYPE(type.flags) == ndCaptureType::NFQ) {
         nd_config_nfq *nfq = static_cast<nd_config_nfq *>(config);
+        nfq->queue_id = (unsigned)r->GetInteger(section,
+          "queue_id", 0);
         nfq->instances = (unsigned)r->GetInteger(section,
           "queue_instances", 1);
+#ifdef _ND_ENABLE_CONNTRACK_COUNTERS
+        nfq->conntrack_counters = conntrack_counters;
+#endif
     }
 
     return true;
@@ -1181,6 +1250,8 @@ bool ndGlobalConfig::AddPlugin(const string &filename) {
 void ndGlobalConfig::UpdatePaths(void) {
     path_app_config = path_state_persistent + "/" + ND_CONF_APP_BASE;
 
+    path_proto_config = path_state_persistent + "/" + ND_CONF_PROTO_BASE;
+
     path_cat_config = path_state_persistent + "/" + ND_CONF_CAT_BASE;
 
     path_legacy_config = path_state_persistent + "/" +
@@ -1197,6 +1268,8 @@ void ndGlobalConfig::UpdatePaths(void) {
     path_functions = path_shared_data + "/" + ND_FUNCTIONS_BASE;
 
     path_interfaces = path_state_persistent + "/" + ND_INTERFACES_BASE;
+
+    path_intel_config = path_state_persistent + "/" + ND_CONF_INTEL_BASE;
 }
 
 void ndGlobalConfig::UpdateConfigVars(void) {
@@ -1208,6 +1281,8 @@ void ndGlobalConfig::UpdateConfigVars(void) {
       ndGC.path_state_volatile));
     conf_vars.insert(
       make_pair("${path_app_config}", path_app_config));
+    conf_vars.insert(
+      make_pair("${path_proto_config}", path_proto_config));
     conf_vars.insert(
       make_pair("${path_category_config}", path_cat_config));
     conf_vars.insert(make_pair("${path_plugins}", path_plugins));
@@ -1215,6 +1290,8 @@ void ndGlobalConfig::UpdateConfigVars(void) {
       make_pair("${path_plugin_libdir}", path_plugin_libdir));
     conf_vars.insert(
       make_pair("${path_categories}", path_categories));
+    conf_vars.insert(
+      make_pair("${path_intelligence_config}", path_intel_config));
     conf_vars.insert(
       make_pair("${path_interfaces}", path_interfaces));
 }
